/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include "gwf_aboutbox.h"

static void		gwf_aboutbox_class_init		(GwfAboutBoxClass	*klass);
static void		gwf_aboutbox_init		(GwfAboutBox		*box);
static void		gwf_aboutbox_destroy		(GtkObject		*object);
static void		on_btnOk_clicked		(GtkButton		*button,
				 		 	gpointer		user_data);

guint
gwf_aboutbox_get_type (void)
{
  static guint gwf_aboutbox_type = 0;

  if (!gwf_aboutbox_type)
    {
      GtkTypeInfo gwf_aboutbox_info =
      {
	"GwfAboutBox",
	sizeof (GwfAboutBox),
	sizeof (GwfAboutBoxClass),
	(GtkClassInitFunc) gwf_aboutbox_class_init,
	(GtkObjectInitFunc) gwf_aboutbox_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };

      gwf_aboutbox_type = gtk_type_unique (GTK_TYPE_WINDOW,&gwf_aboutbox_info);
    }

  return gwf_aboutbox_type;
}

static void		
gwf_aboutbox_class_init (GwfAboutBoxClass *klass)
{
  ((GtkObjectClass *)klass)->destroy = gwf_aboutbox_destroy;
}

static void
gwf_aboutbox_init (GwfAboutBox *box)
{
  GtkWidget	*vbox1;
  GtkWidget 	*vbox2;
  GtkWidget 	*label1;
  GtkWidget 	*hseparator1;
  GtkWidget 	*hbox1;
  GtkWidget 	*btnOk;
  gchar		*ver;

  gtk_widget_set_usize (GTK_WIDGET(box), 250, 250);
  gtk_window_set_title (GTK_WINDOW(box), "About Gwf");

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (box), vbox1);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), vbox2, TRUE, TRUE, 0);

  label1 = gtk_label_new ("Gtk+ Widget Factory");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox2), label1, FALSE, FALSE, 10);

  label1 = gtk_label_new ("A GTK+ Widget Builder");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox2), label1, FALSE, FALSE, 10);

  ver = g_strdup_printf("Version %s",VERSION);
  label1 = gtk_label_new (ver);
  g_free(ver);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox2), label1, FALSE, FALSE, 10);

  label1 = gtk_label_new ("By Jeroen Benckhuijsen");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox2), label1, FALSE, FALSE, 10);

  label1 = gtk_label_new ("Email: jfbenck@gwf.sourceforge.net");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox2), label1, FALSE, FALSE, 0);

  label1 = gtk_label_new ("Web: http://gwf.sourceforge.net");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox2), label1, FALSE, FALSE, 0);

  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox1), hseparator1, TRUE, TRUE, 0);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 5);

  box->button = btnOk = gtk_button_new_with_label ("Ok");
  gtk_widget_show (btnOk);
  gtk_box_pack_start (GTK_BOX (hbox1), btnOk, TRUE, FALSE, 5);
  gtk_widget_set_usize (btnOk, 60, -2);
  GTK_WIDGET_SET_FLAGS (btnOk, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (btnOk), "clicked",
                      GTK_SIGNAL_FUNC (on_btnOk_clicked),
                      (gpointer)box);

  gtk_widget_grab_focus (btnOk);
  gtk_widget_grab_default (btnOk);
}

GtkWidget*
gwf_aboutbox_new(void)
{
  return GTK_WIDGET( gtk_type_new (gwf_aboutbox_get_type()));
}

static void
gwf_aboutbox_destroy (GtkObject *object)
{
  gtk_signal_disconnect_by_data(GTK_OBJECT(GWF_ABOUTBOX(object)->button),(gpointer)object);
}

static void
on_btnOk_clicked			(GtkButton	*button,
   					 gpointer	user_data)
{
  gtk_widget_destroy(GTK_WIDGET(user_data));
}

