/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "gwf_message.h"
#include "dialog.h"
#include <gtk/gtk.h>
#include <gwf_lib.h>

static void ok_button_callback 		(GtkButton *button, gpointer user_data);
static void cancel_button_callback 	(GtkButton *button, gpointer user_data);
static void yes_button_callback 	(GtkButton *button, gpointer user_data);
static void no_button_callback 		(GtkButton *button, gpointer user_data);

void
gwf_message_warning (gchar *message)
{
  GtkWidget	*dialog;
  
  dialog = dialog_with_pixmap_label_create("Gwf Warning!",(gchar**)warning_xpm,message);
  dialog_add_button(dialog,"Ok",GTK_SIGNAL_FUNC(dialog_destroy_callback),NULL);
  gtk_widget_show(dialog);
  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
  gtk_signal_connect(GTK_OBJECT(dialog),"destroy",(GtkSignalFunc)dialog_destroy_gtk_main,NULL);

  gtk_main();
}

void
gwf_message_error (gchar *message)
{
  GtkWidget	*dialog;
  
  dialog = dialog_with_pixmap_label_create("Gwf Error!",(gchar**)error_xpm,message);
  dialog_add_button(dialog,"Ok",GTK_SIGNAL_FUNC(dialog_destroy_callback),NULL);
  gtk_widget_show(dialog);
  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
  gtk_signal_connect(GTK_OBJECT(dialog),"destroy",(GtkSignalFunc)dialog_destroy_gtk_main,NULL);

  gtk_main();
}

void
gwf_message_information (gchar *message)
{
  GtkWidget	*dialog;
  
  dialog = dialog_with_pixmap_label_create("Gwf Information!",(gchar**)information_xpm,message);
  dialog_add_button(dialog,"Ok",GTK_SIGNAL_FUNC(dialog_destroy_callback),NULL);
  gtk_widget_show(dialog);
  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
  gtk_signal_connect(GTK_OBJECT(dialog),"destroy",(GtkSignalFunc)dialog_destroy_gtk_main,NULL);

  gtk_main();
}

GwfChoice
gwf_message_choice (gchar *message,GwfChoice buttons)
{
  GwfChoice	retval = GWF_CANCEL;
  GtkWidget	*dialog;
 
  dialog = dialog_with_pixmap_label_create("Gwf Error!",(gchar**)question_xpm,message);
  
  if (buttons & GWF_OK)
    dialog_add_button(dialog,"Ok",GTK_SIGNAL_FUNC(ok_button_callback),(gpointer)&retval);
  if (buttons & GWF_YES)
    dialog_add_button(dialog,"Yes",GTK_SIGNAL_FUNC(yes_button_callback),(gpointer)&retval);
  if (buttons & GWF_NO)
    dialog_add_button(dialog,"No",GTK_SIGNAL_FUNC(no_button_callback),(gpointer)&retval);
  if (buttons & GWF_CANCEL)
    dialog_add_button(dialog,"Cancel",GTK_SIGNAL_FUNC(cancel_button_callback),(gpointer)&retval);
  
  gtk_widget_show(dialog);
  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
  gtk_signal_connect(GTK_OBJECT(dialog),"destroy",(GtkSignalFunc)dialog_destroy_gtk_main,NULL);

  gtk_main();
  
  return retval;
}

static void
ok_button_callback (GtkButton *button, gpointer user_data)
{
  *((GwfChoice*)user_data) = GWF_OK;
  dialog_destroy_callback(button,NULL);
}

static void
cancel_button_callback (GtkButton *button, gpointer user_data)
{
  *((GwfChoice*)user_data) = GWF_CANCEL;
  dialog_destroy_callback(button,NULL);
}

static void
yes_button_callback (GtkButton *button, gpointer user_data)
{
  *((GwfChoice*)user_data) = GWF_YES;
  dialog_destroy_callback(button,NULL);
}

static void
no_button_callback (GtkButton *button, gpointer user_data)
{
  *((GwfChoice*)user_data) = GWF_NO;
  dialog_destroy_callback(button,NULL);
}

