/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef GWF_TREEDATA_H
#define GWF_TREEDATA_H

#ifdef __cplusplus
extern "C" {
#endif

#include <gtk/gtk.h>
#include "gwf_project.h"

typedef struct	_GwfTreeData		GwfTreeData;
typedef struct	_GwfTreeDataList	GwfTreeDataList;
typedef struct	_GwfPTData		GwfPTData;		
typedef void	(*GwfTreeDataAttachFunc)	(GwfTreeData *data,GwfProject *project);
typedef void	(*GwfTreeDataFreeFunc)		(GwfTreeData *data);
typedef void	(*GwfTreeDataForeachFunc)	(GwfTreeData *data,gpointer user_data);

struct _GwfTreeData
{
   gchar			*name;
   GtkCTree			*tree;
   GwfTreeDataAttachFunc	attach;
   GwfTreeDataFreeFunc		free;
   GPtrArray			*ptdata;
   gint				nnodes;
};

struct _GwfTreeDataList
{
  GHashTable	*datas;
};

struct _GwfPTData
{
  GwfTreeData	*data;
  gchar		*node;
  GtkCTreeNode	*parent;
  gchar		*signame;
  gint		sigid;
  GtkSignalFunc	func;
  gpointer	user_data;
  GwfProject	*project;
};

GwfTreeDataList*	gwf_treedata_list_new		(void);
void			gwf_treedata_list_free		(GwfTreeDataList *list);
gint			gwf_treedata_list_add		(GwfTreeDataList *list,GwfTreeData *data);
void			gwf_treedata_list_remove	(GwfTreeDataList *list,gchar *name);
GwfTreeData*		gwf_treedata_list_get_data	(GwfTreeDataList *list,gchar *name);
void			gwf_treedata_list_foreach	(GwfTreeDataList *list,GwfTreeDataForeachFunc func,gpointer user_data);

GwfTreeData*		gwf_treedata_new		(gchar *name);
void			gwf_treedata_free		(GwfTreeData *data);
void			gwf_treedata_set_attach		(GwfTreeData *data,GwfTreeDataAttachFunc func);
void			gwf_treedata_set_free		(GwfTreeData *data,GwfTreeDataFreeFunc func);

void			gwf_treedata_attach		(GwfTreeData *data,GwfProject *project,GtkCTree *tree);
void			gwf_treedata_free_data		(GwfTreeData *data);

GwfPTData**		gwf_treedata_register_ptdata		(GwfTreeData *data,GwfPTData def[],gint nnodes,GtkCTreeNode *parent,GwfProject *project);
void			gwf_treedata_unregister_ptdata_by_node	(GwfTreeData *data,GtkCTreeNode *node);
void			gwf_treedata_unregister_all_ptdata	(GwfTreeData *data);



void			gwf_projtree_remove_node_ptdata		(GwfTreeData *data,GtkCTreeNode *node);

GwfPTData**		gwf_projtree_add_new_section 		(GwfTreeData *data,GwfProject *project,gchar *secname,GwfPTData def[],gint nnodes);
void			gwf_projtree_insert_string_by_ptdata	(GwfPTData *ptdata,gchar *value);
void			gwf_projtree_insert_bool_by_ptdata	(GwfPTData *ptdata,gboolean value);
void			gwf_projtree_insert_float_by_ptdata	(GwfPTData *ptdata,gfloat value);
void			gwf_projtree_insert_int_by_ptdata	(GwfPTData *ptdata,glong value);

void			gwf_projtree_node_string_changed_cb 	(GwfProject *project,gchar *text,gpointer user_data);
void			gwf_projtree_node_bool_changed_cb 	(GwfProject *project,gboolean bool,gpointer user_data);
void			gwf_projtree_node_int_changed_cb 	(GwfProject *project,glong value,gpointer user_data);
void			gwf_projtree_node_float_changed_cb 	(GwfProject *project,gfloat value,gpointer user_data);

#ifdef __cplusplus
}
#endif

#endif


