/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gwf_lib.h>
#include "gwf_xmlio_elem.h"
#include "gwf_projdata_elem.h"
#include <gtk/gtk.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

enum
{
  GWF_ELEMENT,
  GWF_ELEMENT_NAME,
  GWF_ELEMENT_TYPE,
  GWF_ELEMENT_VALUE,
  GWF_ELEMENT_SET_FUNC,
  GWF_ELEMENT_SETALLOC,
  GWF_ELEMENT_SETCODE,
  GWF_ELEMENT_GET_FUNC,
  GWF_ELEMENT_GETALLOC,
  GWF_ELEMENT_GETCODE,
  LAST_ELEMENT_TAG
};

static gint		gwf_xmlio_parse_element		(xmlNodePtr cur,xmlNsPtr ns,GwfXmlIO *xml_io,GwfProject *project,GwfError *error);
static void		gwf_xmlio_add_element_foreach 	(element* elem,gpointer user_data);

static gchar		*xml_tags[LAST_ELEMENT_TAG] = {
							"ELEMENT",
        						"NAME",
               						"TYPE",
							"ELEMENTVALUE",
   							"ELEMENTSETFUNC",
        						"ALLOC",
   							"ELEMENTSETCODE",
   							"ELEMENTGETFUNC",
                                                        "ALLOC",
                                                        "ELEMENTGETCODE"
                                                       };


gint
gwf_xmlio_load_element(xmlNodePtr cur,xmlNsPtr ns,GwfXmlIO *xml_io,GwfProject *project,GwfError *error)
{
  gint		ok = 1;

  g_return_val_if_fail(cur != NULL,0);
  g_return_val_if_fail(xml_io != NULL,0);
  g_return_val_if_fail(project != NULL,0);
  g_return_val_if_fail(error != NULL,0);

  while (cur && ok)
  {
    if ((strcmp(cur->name,xml_tags[GWF_ELEMENT]) == 0) && (cur->ns == ns))
      ok = gwf_xmlio_parse_element(cur,ns,xml_io,project,error);
    else
    {
      gwf_error_raise_error(error,"XmlIO : Unknown tag",cur->name);
      return 0;
    }
    cur = cur->next;
  }

  return ok;
}

gint
gwf_xmlio_save_element(xmlNodePtr parent,xmlNsPtr ns,GwfProject *project)
{
  XmlSaveData		save_data;

  save_data.ns = ns;
  save_data.project = project;
  save_data.parent = parent;
  save_data.result = 1;

  gwf_project_element_foreach(project,gwf_xmlio_add_element_foreach,(gpointer)&save_data);

  return save_data.result;
}

static gint
gwf_xmlio_parse_element(xmlNodePtr cur,xmlNsPtr ns,GwfXmlIO *xml_io,GwfProject *project,GwfError *error)
{
  element	elem;
  gchar		*tmp;

  g_return_val_if_fail(cur != NULL,0);
  g_return_val_if_fail(xml_io != NULL,0);
  g_return_val_if_fail(project != NULL,0);
  g_return_val_if_fail(error != NULL,0);

  elem.name = xmlGetProp(cur,xml_tags[GWF_ELEMENT_NAME]);
  if (!elem.name)
  {
    gwf_error_raise_error(error,"XmlIO : Element no name");
    return 0;
  }

  elem.type = xmlGetProp(cur,xml_tags[GWF_ELEMENT_TYPE]);

  cur = cur->childs;

  while (cur)
  {
    if ((strcmp(cur->name,xml_tags[GWF_ELEMENT_VALUE]) == 0)  && (cur->ns == ns))
    {
      elem.value = xmlNodeGetContent(cur);
    }
    else
    if ((strcmp(cur->name,xml_tags[GWF_ELEMENT_SET_FUNC]) == 0) && (cur->ns == ns))
    {
      tmp = xmlNodeGetContent(cur);

      /* For compatibility with file version 0.1.0
       * in which the set_func element was a string.
       * If the string contains a value set_func is set to TRUE
       * Otherwise the string is analysed
      */
      elem.set_func = (tmp && (strcmp(tmp,"FALSE") != 0) && (strcmp(tmp,"TRUE") != 0)) ? TRUE : STR_TO_BOOL(tmp);

      g_free(tmp);
      tmp = xmlGetProp(cur,xml_tags[GWF_ELEMENT_SETALLOC]);
      elem.alloc_set_func = (tmp) ? STR_TO_BOOL(tmp) : FALSE;
      g_free(tmp);

      elem.set_code = (cur->childs->next && (strcmp(cur->childs->next->name,xml_tags[GWF_ELEMENT_SETCODE]) == 0) && (cur->ns == ns) ) ?
                       xmlNodeGetContent(cur->childs->next) : NULL;
    }
    else
    if ((strcmp(cur->name,xml_tags[GWF_ELEMENT_GET_FUNC]) == 0) && (cur->ns == ns))
    {
      tmp = xmlNodeGetContent(cur);

      /* For compatibility with file version 0.1.0
       * in which the get_func element was a string.
       * If the string contains a value get_func is set to TRUE
       * Otherwise the string is analysed
      */
      elem.get_func = (tmp && (strcmp(tmp,"FALSE") != 0) && (strcmp(tmp,"TRUE") != 0)) ? TRUE : STR_TO_BOOL(tmp);

      g_free(tmp);
      tmp = xmlGetProp(cur,xml_tags[GWF_ELEMENT_GETALLOC]);
      elem.alloc_get_func = (tmp) ? STR_TO_BOOL(tmp) : FALSE;
      g_free(tmp);

      elem.get_code = (cur->childs->next && (strcmp(cur->childs->next->name,xml_tags[GWF_ELEMENT_GETCODE]) == 0) && (cur->ns == ns) ) ?
                       xmlNodeGetContent(cur->childs->next) : NULL;
    }
    else
    {
      gwf_error_raise_error(error,"XmlIO : Unknown tag",cur->name);
      return 0;
    }

    cur = cur->next;
  }

  gwf_project_add_element(project,&elem);
  g_free(elem.name);
  g_free(elem.type);
  g_free(elem.value);

  return 1;
}

static void
gwf_xmlio_add_element_foreach (element *elem,gpointer user_data)
{
  xmlNodePtr	elm,tmp;
  XmlSaveData	*data = user_data;

  elm = xmlNewChild(data->parent,data->ns,xml_tags[GWF_ELEMENT],NULL);
  xmlSetProp(elm,xml_tags[GWF_ELEMENT_NAME],elem->name);
  xmlSetProp(elm,xml_tags[GWF_ELEMENT_TYPE],elem->type);

  xmlNewChild(elm,data->ns,xml_tags[GWF_ELEMENT_VALUE],elem->value);

  tmp = xmlNewChild(elm,data->ns,xml_tags[GWF_ELEMENT_SET_FUNC],BOOL_TO_STR(elem->set_func));
  xmlSetProp(tmp,xml_tags[GWF_ELEMENT_SETALLOC],BOOL_TO_STR(elem->alloc_set_func));

  if (elem->set_code)
    xmlNewChild(tmp,data->ns,xml_tags[GWF_ELEMENT_SETCODE],elem->set_code);

  tmp = xmlNewChild(elm,data->ns,xml_tags[GWF_ELEMENT_GET_FUNC],BOOL_TO_STR(elem->get_func));
  xmlSetProp(tmp,xml_tags[GWF_ELEMENT_GETALLOC],BOOL_TO_STR(elem->alloc_get_func));

  if (elem->get_code)
    xmlNewChild(tmp,data->ns,xml_tags[GWF_ELEMENT_GETCODE],elem->get_code);
}
